import abc
import typing
import asyncio


class AbsDriver(metaclass=abc.ABCMeta):

    handler: typing.Callable

    loop: asyncio.AbstractEventLoop

    @abc.abstractmethod
    async def run(self, **kwargs):
        pass

    @abc.abstractmethod
    async def kill(self):
        pass


import asyncio

import quart

from . import model


class HTTPDriver(model.AbsDriver):
    
    def __init__(self, handler: callable):
        self.handler = handler

    async def run(self, **kwargs):
        app = quart.Quart(__name__)
        
        # 监听所有的 HTTP 请求
        @app.route('/', defaults={'path': ''}, methods=['GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'OPTIONS'])
        @app.route('/<path:path>', methods=['GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'OPTIONS'])
        async def catch_all(path: str):
            req = {
                'method': quart.request.method,
                'path': path,
                'headers': quart.request.headers,
                'body': await quart.request.get_data()
            }
            resp = await self.handler(req)

            return quart.Response(
                status=resp['status'],
                headers=resp['headers'],
                response=resp['body']
            )

        # 启动 HTTP 服务
        host = kwargs.get('host', '0.0.0.0')
        port = kwargs.get('port', 8181)
        debug = kwargs.get('debug', False)

        self.loop = asyncio.get_event_loop()

        return await app.run_task(host, port, debug=debug)

    async def kill(self):
        self.loop.stop()


import typing
import asyncio

import websockets

from . import model


class WSDriver(model.AbsDriver):

    handler: typing.Callable[[websockets.WebSocketServerProtocol, str], typing.Any]

    task: asyncio.Task

    def __init__(self, handler: typing.Callable[[websockets.WebSocketServerProtocol, str], typing.Any]):
        self.handler = handler

    async def run(self, **kwargs):
        self.task = asyncio.create_task(self._run(**kwargs))
        return await self.task

    async def _run(self, **kwargs):
        host = kwargs.get('host', '0.0.0.0')
        port = kwargs.get('port', 8182)
        async with websockets.serve(self.handler, host, port):
            try:
                await asyncio.Future()
            except asyncio.CancelledError:
                pass

    async def kill(self):
        self.task.cancel()


import typing

import pydantic

from . import connection


class Bot(pydantic.BaseModel):
    """
    机器人实体信息
    """

    account_id: str
    """账号ID"""

    nickname: str
    """昵称"""

    connection_types: typing.List[connection.ConnectionType]
    """连接类型"""



import asyncio
from typing import List
import uuid
import typing
import logging
import json
import re

import websockets

from ..driver import model as driver_model, http, ws
from ..entity import bot, connection
from . import model


class MiraiAPIHTTPAdapter(model.AbsPlatformAdapter):
    
    forward_ws: ws.WSDriver
    """正向WebSocket驱动器"""

    sessions: list[dict]
    """会话列表
    [
        {
            "session_key": "uuid",
            "bot": bot.Bot,
            "connections": {
                "ws": {
                    "path": "/ws",
                    "websocket": websockets.WebSocketServerProtocol,
                }
            }
        }
    ]
    """

    verify_key: str = None

    def __init__(self):
        self.sessions = []

    async def emit_event(self, bots: List[bot.Bot], event: dict):
        for bot in bots:
            await self._ws_send(bot, event, sync_id='-1')

    async def run(self, bots: typing.List[bot.Bot], verify_key: str = None):
        """
        启动服务端，阻塞运行

        :param bots: Bot列表
        :param verify_key: 验证密钥，可选，不设置时不验证
        """
        self.verify_key = verify_key

        async def ws_handler(websocket: websockets.WebSocketServerProtocol, path: str):
            
            # 验证密钥
            verify_key = None
            if 'verifyKey' in websocket.request_headers:
                verify_key = websocket.request_headers['verifyKey']
            else:
                # 取路径中的参数: /ws?verify_key=yirimirai
                extra = re.findall(r'\?verifyKey=(.+)', path)
                if len(extra) > 0:
                    verify_key = extra[0]
            if self.verify_key is not None and verify_key != self.verify_key:
                await self._ws_send_login_msg(websocket, {
                    'code': 1,
                    'msg': '错误的verify key'
                })
                return await websocket.close()

            qq = 0
            if 'qq' in websocket.request_headers:
                qq = int(websocket.request_headers['qq'])
            else:
                # 取路径中的参数: /ws?qq=12345678
                extra = re.findall(r'\?qq=(\d+)', path)
                if len(extra) > 0:
                    qq = int(extra[0])

            bot_obj = None

            for bot in bots:
                if bot.account_id == str(qq):
                    bot_obj = bot
                    break
            if bot_obj is None:
                await self._ws_send_login_msg(websocket, {
                    'code': 1,
                    'msg': '指定的Bot不存在'
                })
                return await websocket.close()
            else:
                # 存放会话
                session_key = str(uuid.uuid4())
                self.sessions.append({
                    'session_key': session_key,
                    'bot': bot_obj,
                    'connections': {
                        'ws': {
                            'path': path,
                            'websocket': websocket
                        }
                    }
                })

                # 发送会话密钥
                await self._ws_send_login_msg(websocket, {
                    'code': 0,
                    'session': session_key
                })

                async for message in websocket:
                    if self._action_handler is None:
                        continue
                    await self._action_handler(bot_obj, connection.ConnectionType.FORWARD_WS, message)

        self.forward_ws = ws.WSDriver(ws_handler)

        return await self.forward_ws.run()

    async def _ws_send_login_msg(self, websocket: websockets.WebSocketServerProtocol, data: dict):
        return await websocket.send(json.dumps({
            'syncId': '',
            'data': data
        }, ensure_ascii=False))

    async def _ws_send(self, bot: bot.Bot, data: dict, sync_id: str = '') -> bool:
        sent = False
        for session in self.sessions:
            if session['bot'] == bot:
                await session['connections']['ws']['websocket'].send(json.dumps({
                    'syncId': sync_id,
                    'data': data
                }, ensure_ascii=False))
                sent = True
                break

        return sent

    async def send(self, bot: bot.Bot, data: dict, sync_id: str = '', connection_type: connection.ConnectionType=connection.ConnectionType.FORWARD_WS) -> bool:
        return await self._ws_send(bot, data, sync_id)

    async def kill(self):
        for session in self.sessions:
            await session['connections']['ws']['websocket'].close()

        await self.forward_ws.kill()



from setuptools import find_namespace_packages
from setuptools import setup

setup(
    name="skittles",
    version="0.1.1",
    description="mirai-api-http Mock 测试工具",
    long_description=open("README.md", "rt", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/RockChinQ/Skittles",
    project_urls={
        "Bug Report": "https://github.com/RockChinQ/Skittles/issues"
    },
    author="RockChinQ",
    author_email="1010553892@qq.com",
    license="GNU Affero General Public License v3.0",
    packages=find_namespace_packages(".", exclude="tests"),
    package_dir={"": "."},
    py_modules=["skittles"],
    package_data={"": ["*.json"]},
    install_requires=[
        "websockets",
        "pydantic",
        "quart",
        "aiocqhttp",
    ],
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Natural Language :: Chinese (Simplified)",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)


import abc
import typing

from ..entity import bot, connection


class AbsPlatformAdapter(metaclass=abc.ABCMeta):

    _action_handler: typing.Callable[
        [bot.Bot, connection.ConnectionType, str],
        typing.Coroutine[typing.Any, typing.Any, typing.Any]
    ]
    """动作处理器"""

    def action_handler(self, handler: typing.Callable[
        [bot.Bot, connection.ConnectionType, str],
        typing.Coroutine[typing.Any, typing.Any, typing.Any]
    ]):
        """
        设置 API 调用处理器
        当客户端发起一个动作（调用API）时，平台服务端实现将接收到的数据直接发给这个处理器
        """
        self._action_handler = handler
    
    @abc.abstractmethod
    async def emit_event(self, bots: typing.List[bot.Bot], event: dict):
        """
        发送事件
        通过这个方法，调用平台服务端，将事件数据dict发送给客户端
        """
        pass

    @abc.abstractmethod
    async def run(self, bots: typing.List[bot.Bot], **kwargs):
        pass

    @abc.abstractmethod
    async def kill(self):
        pass